<?php

declare(strict_types=1);

namespace apexl\Io\modules\user\Factories;

use apexl\Io\factories\EntityFactory;
use apexl\Io\modules\user\entities\userEntity;

/**
 * @method userEntity create(array $attrs = [], bool $store = true)
 */
final readonly class UserEntityFactory extends EntityFactory
{

}